///***************************************************************************************
// BMP280/BME280 Pressure Sensor
// Do I2C Bit Banger
// Jed Margolin  2/1/2021.
// Built with Code Composer Studio v10
//***************************************************************************************
#include <msp430.h>
#include <math.h>
#include "bme280.h"
//================================================
// Read i2C byte
unsigned char i2c_read(unsigned char address)
{
    unsigned char ack;

// SDA is an output and high

// Start bit
    i2c_start_bit();

// send the ID
   ack = i2c_sendbyte(IIC_ID_WRITE);

// we will keep going even if ack is high
   ack = ack | i2c_sendbyte(address);

// How does the slave send the data back?

// Repeat the start bit
   i2c_start_bit();

// Send the device ID
   ack = ack | i2c_sendbyte(IIC_ID_READ);

   ack = ack | i2c_getbyte();     // data ic2_rxdata is global

// Give it a Stop bit
   i2c_stop_bit();

   return(i2c_rxdata);      // global
}
//=====================================
void i2c_start_bit()
{
// Make sure SCL is low since an SDA transition with SCL low will be ignored
   SCL_POUT &= ~SCL_BIT;

// Make SDA high
   SDA_POUT |= SDA_BIT;
   wait(7);

   SCL_POUT |= SCL_BIT;   // SCL high
   wait(5);
   SDA_POUT &= ~SDA_BIT;   //SDA Low
   wait(2);

   SCL_POUT &= ~SCL_BIT;   //SCL low
   wait(2);
}
//===================================================
void i2c_stop_bit()
{
// Make sure SCL is low since an SDA transition with SCL low will be ignored
   SCL_POUT &= ~SCL_BIT;
   wait(4);

// Make SDA low
   SDA_POUT &= ~SDA_BIT;
   wait(8);

// Make SCL high
   SCL_POUT |= SCL_BIT;   // SCL high

   wait(8);
   SDA_POUT |= SDA_BIT;   //SDA high

   wait(10);
}
//=================================================
unsigned char i2c_sendbyte(unsigned char data)
{
    unsigned char n, ack;

    for(n=8; n>0; n--)
    {
       if(data & 0x80)
       {
          SDA_POUT |= SDA_BIT;   // SDA high
       }
       else {SDA_POUT &= ~SDA_BIT;}  // SDA low
       data = data<<1;

       wait(6);
       SCL_POUT |= SCL_BIT;   //SCL high
       wait(7);
       SCL_POUT &= ~SCL_BIT;   //SCL low
//       wait(10);
    }

// look for ACK from I2C device
    SDA_PDIR &= ~SDA_BIT;   // Input

    wait(4);
    SCL_POUT |= SCL_BIT;   //SCL high

// ack is immediately available
    if(SDA_PIN & SDA_BIT)
    {
        ack = 1;
    }
    else {ack = 0;}

    wait(8);

// Then make SCL low
    SCL_POUT &= ~SCL_BIT;   //SCL low

// Give the slave time to release SDA
    wait(100);

// Then make SDA an output again and low; SCL is low so transitions won't matter
    SDA_POUT &= ~SDA_BIT;
    SDA_PDIR |= SDA_BIT;     // An output again

    wait(10);

// Then SDA low
//    P1OUT &= ~BIT7;     //SDA low
//    wait(10);

    return(ack);
}
//=======================================
unsigned char i2c_getbyte()
{
    unsigned char n, data;

// Make SDA an input
//    P1SEL  &= ~BIT7;  // In
//    P1SEL2 &= ~BIT7;
    SDA_PDIR &= ~SDA_BIT;   // In

    data = 0;

    for(n=8; n>0; n--)
    {
       data = data<<1;
       if(SDA_PIN & SDA_BIT)
       {
          data = data | 0x01;   // SDA high
       }

       wait(6);
       SCL_POUT |= SCL_BIT;   //SCL high
       wait(7);
       SCL_POUT &= ~SCL_BIT;   //SCL low

    }
//       wait(7);

// We are responsible for sending ACK or NACK
// so make SDA an output
    SDA_PDIR |= SDA_BIT;     //Out/

// An ACK tells the slave to keep sending data, a NACK says no more
// We are only asking for one byte here so send a NACK
    SDA_POUT |= SDA_BIT;

    wait(4);
    SCL_POUT |= SCL_BIT;   //SCL high
    wait(10);
    SCL_POUT &= ~SCL_BIT;   //SCL low

    SDA_POUT |= SDA_BIT;  // SDA back to high to get ready for the stop bit

// and we are done
    i2c_rxdata = data;
    return(0);
}
//====================================
int wait_ready()
{
    int n, timeout;
    unsigned char busy, data;

    busy = 1;
    n = 1000;
    timeout = 0;

    while(busy)
    {
       data = i2c_read(0xF3);
       if( !(data & 0x08)) {busy = 0;}
       if(n == 0)  {timeout = 1;}
       n--;
    }
    return timeout;
}
//==================================
void wait(int i)
{
   unsigned int n;

   for(n=i; n>0; n--)
   {
     asm(" nop");
   }
}
//=======================================================
void bme_init(void)
{
    unsigned char address;

// BME280 reset
    address = bme_reset;
    i2c_write(address,0);

    wait1();

// Initialize the BME-280
// config    d7   d6   d5 | d4   d3   d2 | d1 | d0
//              t_sb          filter        x   spi3W_en
//            0    0    0    0    0    0    0    0
// t_sb = 000 (0.5 ms  standby time
// filter = 000  filter off
    address = bme_config;
    i2c_write(address,0x00);        // 0x50

// humidity: oversampling x1   ('0' = off)
    i2c_write(0xF2,0x01);

// ctrl_meas    d7   d6   d5 |  d4   d3   d2 | d1   d0
//                 osrs_t           osrs_p       mode
//               0    0    1     0    1    1   0     1
// osrs_t = 001  oversampling x 1 17-bit  ('0' = off)
// osrs_p = 011 oversampling x4   ('0' = off)
// mode = 11 (normal)  (0 = sleep; 01 and 10 = Forced Mode)
    address = bme_ctrl_meas;
    i2c_write(address,0x2D);  // In forced mode it must be called for every reading
    wait1();
    wait_ready();
}
//==============================================
// Get the calibration values
void getcal()
{
    unsigned char rxtemp;
    int k;

    rxtemp = 0x88;      // temperature calibration data
    for(k=0; k<6; k++)
    {
        wait_ready();
        caldata[k]= i2c_read(rxtemp++);
    }

// Temperature

// Make digs for compensation functions. All digs are global
//Requires that the calata array be unsigned
// caldata[0] = dig_T1 lsb  caldata[1] = dig_T1 msb unsigned
   dig_T1 = caldata[1]<<8 | caldata[0];

// caldata[2] = dig_T2 lsb  caldata[3] = dig_T2 msb signed
   dig_T2 = caldata[3]<<8 | caldata[2];

// caldata[4] = dig_T3 lsb  caldata[5] = dig_T3 msb signed
   dig_T3 = caldata[5]<<8 | caldata[4];

// Humidity
   wait_ready();
   humdata[0]= i2c_read(0xA1);   // for dig_H1 (unsigned char)

   wait_ready();
   humdata[1]= i2c_read(0xE1);   // for dig_H2 7:0

   wait_ready();
   humdata[2]= i2c_read(0xE2);   // for dig_H2 15:8

   wait_ready();
   humdata[3]= i2c_read(0xE3);   // for dig_H3 7:0  (unsigned char)

   wait_ready();
   humdata[4]= i2c_read(0xE4);   // for dig_H4 11:4

   wait_ready();
   humdata[5]= i2c_read(0xE5);   // 3:0 for dig_H4; 7:4 for for dig_H5 3:0

   wait_ready();
   humdata[6]= i2c_read(0xE6);   // for dig_H5 11:4

   wait_ready();
   humdata[7]= i2c_read(0xE7);   // for dig_H6 (signed char)

// make the digs
   // humdata[0] is dig_H3 7:0 (unsigned char);
   dig_H1 = humdata[0];  //  7:0

   // humdata[1] is dig_H2 7:0; humdata[2] is dig_H2 8:15  (result is )
   dig_H2 = (humdata[2]<<8) | humdata[1];

   // humdata[3] is dig_H3 7:0  (unsigned char);
   dig_H3 = humdata[3];   // dig_H3 7:0

   // humdata[4] (7:0) is dig_H4 11"4; humdata[5] (3:0) is dig_H4 3:0
   dig_H4 = (humdata[4] <<4) | (humdata[5] & 0x0F);

   // humdata[5] 7:4 is also dig_H5 3:0; humdata[6] is dig_H5 11:4
   dig_H5 = (humdata[5]>>4) | (humdata[6]<<4);

   dig_H6 = humdata[6];     // signed char
}
//=====================================================
// Read the BME280 data for temperature, and humidity
void read_bme280(void)
{
   unsigned int k;
   unsigned long hum1, hum2;
   unsigned char address,rxtemp;

// humidity: oversampling x1   ('0' = off)
   i2c_write(0xF2,0x01);
   address = bme_ctrl_meas;
   i2c_write(address,0x2D);  // In forced mode it must be called for every reading
   wait5ms();

// Read temperature data
   rxtemp = 0xFA;
   wait_ready();
   for(k=0; k<3; k++)
   {
      wait_ready();
      bmedata[k]= i2c_read(rxtemp++);
   }

   bmedata[2] &= 0x0F;     // temp_xlsb, temperature is maximum of 20 bits

   adc_t = 0;               // 0 - msb, 1 = lsb, 2 = xlsb
   adc_t = bmedata[0];

   adc_t = adc_t<<8;        // 8
   adc_t = adc_t | bmedata[1];

   adc_t = adc_t<<8;        // 3
   adc_t = adc_t | bmedata[2];
   adc_t = adc_t>>4;

// Humidity
   hum1 = ( (unsigned long)i2c_read(0xFD)) <<8;       // msb
   hum2 = (unsigned long)i2c_read(0xFE);
   adc_h = hum1 | hum2;

   do_compensation();       // F is temperature in Fahrenheit; humidity is in %
   asm(" nop");    // for debugger
}
//==========================================================
// Compensation for Temperature, Pressure, and Humidity
int do_compensation()
{
    long var1, var2;
    long var3, var4, var5;

    long t_fine;

// Temperature Compensation
    var1 = ((((adc_t>>3) - ((long)dig_T1<<1))) * ((long)dig_T2)) >> 11;
    // var1 = 128793

    var2 = (((((adc_t>>4) - ((long)dig_T1)) * ((adc_t>>4) - ((long)dig_T1))) >> 12) * ((long)dig_T3)) >> 14;
    // var2 = -371

    t_fine = var1 + var2;
    // t_ fine = 128422

    T = (t_fine * 5 + 128) >> 8;
    // T = 2508 = 25.08 deg C

    F = T * 9 / 5 + 3200;
    // F = 7714

// Humidity Compensation
// Bosch code
//  Returns humidity in %RH as unsigned 32 bit integer in Q22.10 format (22 integer and 10 fractional bits).
//  Output value of 47445" represents 47445/1024 = 46.333 %RH

// From the datasheet
/*
    var = (t_fine - ( (signed long)76800) );

    var = ( ( ( ( (adc_h << 14) - ( ( (signed long)dig_H4) << 20) - ( ( (signed long)dig_H5) *
       var))  + ( (signed long)16384) )  >> 15) * ( ( ( ( ( ( (var  *
       ( (signed long)dig_H6)) >> 10)  *  ( ( (var * ( (signed long)dig_H3)) >> 11) +
       ( (signed long)32768) )) >> 10)  +  ((signed long)2097152)) * ((signed long)dig_H2) +
       8192) >> 14));

    var = (var  - ( ( ( ( (var >> 15) * (var >> 15)) >> 7) *  ((signed long)dig_H1) ) >> 4));
    var = (var < 0 ? 0 : var);
    var = (var > 419430400 ? 419430400 : var);

    Humidity = ((unsigned long)var>>12)/1000 ;  // "46332" is 46.332 %
*/

// Bosch Code from https://github.com/BoschSensortec/BME280_driver
// The results are the same as the previous code from the data sheet
    var1 = t_fine - (long)76800;
    var2 = (long)adc_h * 16384;           // adc_h is the uncompensated humidity
    var3 = (long)dig_H4 * 1048576;
    var4 = (long)dig_H5 * var1;
    var5 = ( ((var2 - var3) - var4) + (long)16384 ) / 32768;
    var2 = (var1 * (long)dig_H6 ) / 1024;
    var3 = ( (var1 * (long)dig_H3) ) / 2048;
    var4 = ( (var2 * (var3 + (long)32768)) / 1024) + (long)2097152;
    var2 = ( (var4 * (long)dig_H2 ) + 8192) / 16384;
    var3 = var5 * var2;
    var4 = ((var3 / 32768) * (var3 / 32768)) / 128;
    var5 = var3 - ((var4 * (long)dig_H1 ) / 16);
    var5 = (var5 < 0 ? 0 : var5);
    var5 = (var5 > 419430400 ? 419430400 : var5);
    Humidity = ( (unsigned long)var5 / 4096);

    // mine
    Humidity = Humidity/1000;

    if (Humidity > 99)
    {
       Humidity = 99;
    }

    return 1;
}
//==========================================================
void wait1()
{
volatile unsigned int i;

    i = 0;
    return;
}
//=================================================================
// Write to a register with i2c
unsigned char i2c_write(unsigned char address, unsigned char data)
{
    unsigned char ack;

// Make sure SDA is an output and high
//    P1SEL  &= ~BIT7;   // Out
//    P1SEL2 &= ~BIT7;
//    P1DIR |= BIT7;     //Out
//    P1OUT |= BIT7;

// Start bit
     i2c_start_bit();

// send the ID
    ack = i2c_sendbyte(IIC_ID_WRITE);

// we will keep going even if ack is high
    ack = ack | i2c_sendbyte(address);

// Send the data
    ack = ack | i2c_sendbyte(data);

 // Give it a Stop bit
    i2c_stop_bit();

    return(ack);
 }
//==========================================================
