/*
* MyTX4 transmits the same signals as the La Crosse TX4U wireless sensor (which they don't make anymore)
* 11/15/2021
*
* Copyright 2021  Jed Margolin
* Permission is granted to individuals to make copies for their personal use.
* Any commercial use requires my permission.
*
* Uses Mytx4-rfm69hw-R2 board with MSP430G2402 and others.
* Needs at least 8KB program and 256 Bytes RAM
* Uses the RFM69 transceiver (SPI Interface) for OOK
*
* The unit sends the temperature in Celsius*100 + 5000. The base unit subtracts 5000.
* This is so the temperature value is always positive.
* The base unit also converts the temperature to Fahrenheit (if selected)
*
* Built with Code Composer Studio v10
*
* Read the BME280 and send the temperature and humidity to the base display
* Uses I2C Interface for the BME280
*
* Switch SW2 selects between two sensor addresses (ids)
*
* I2C bits are defined in bme280.h
*
* Use MSP430G2402 or MSP430G2553 (or others)
*
* Compiler Optimization must be set to 0-Register Optimization only

 P1.0 input  switch pad
 P1.1 input  switch pad
 P1.2 output xmit (rfm69 Dio2)
 P1.3 input  Address switch
 P1.4 output /CS  to RFM69
 P1.5 output SCLK to RFM69
 P1.6 output SDOUT to RFM69 SDIN
 P1.7 input  SDIN  to RFM69 SDOUT

 P2.0 input  switch
 P2.1 output test out
 P2.2 output error
 P2.3 input  not used
 P2.4 output I2C SCL for board
 P2.5 input  I2C SDA for board
 P2.6 input  (Not used as Xin)
 P2.7 output (Not used as Xout)  Reset RFM69  ( 1 = Reset)
*/
//==================================================================
#define MSPTYPE 0   // 0 = MSP430G2402, 1 = MSP430G2553
#define TESTLOOP 0

#include <msp430.h>
#include "main.h"
#include "rfm69.h"
#include <bme280.h>

int main(void)
{
   WDTCTL = WDTPW | WDTHOLD;   // stop watchdog timer

// No crystal. Use the factory default values
   DCOCTL = CALDCO_1MHZ;      // factory calibration vqlues
   BCSCTL1 = CALBC1_1MHZ;
   BCSCTL2 = 0x06;            // DIVS (Divider for SMCLK) /8 for 1 sec interrupts

// Port 1 - All Digital I/O
   P1SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
   P1SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 1 - outputs for rfm69
   P1DIR |= (BIT2|BIT4|BIT5|BIT6);  // Out:  Xmit, /CS, SCLK, SDOUT
   P1OUT = BIT4;                    //      /CS
   P1OUT &= ~(BIT2|BIT5|BIT6);      //      Xmit,SCLK, SDOUT

// Port 1 - inputs
   P1REN |= BIT0|BIT1|BIT3|BIT7;     // pullup/pulldown enabled
   P1OUT |= BIT0|BIT1|BIT3|BIT7;     // pullup
   P1DIR &= ~(BIT0|BIT1|BIT3|BIT7);  // In

// Port 2 - All Digital I/O
   P2SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
   P2SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);

// Port 2 - inputs
   P2REN = BIT0|BIT1|BIT2|BIT3|BIT6;      // pullup/pulldown enabled
   P2OUT = BIT0|BIT1|BIT2|BIT3|BIT6;      //
   P2DIR &= ~(BIT0|BIT3);  // In

// Port 2 - outputs
   P2OUT &= ~(BIT4|BIT5|BIT7);
   P2OUT |= BIT4|BIT5|BIT7;     // SCLK and SDA
   P2DIR |= (BIT4|BIT5|BIT7);   // Out

#if MSPTYPE
// Port 3 - inputs (the 2553 die has 3 ports, the PDIP brings out only two)
    P3SEL  &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);  // In
    P3SEL2 &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);
    P3REN |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    // pullup/pulldown enabled
    P3OUT |= (BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);    //
    P3DIR &= ~(BIT0|BIT1|BIT2|BIT3|BIT4|BIT5|BIT6|BIT7);   // In
#endif

// Reset rfm69
   P2OUT |= BIT7;
   waitms(1);
   P2OUT &= ~BIT7;
   waitms(10);

// rfm69
// test
//    rfm69_test1();
//    rfm69_test2();
//    rfm69_test3();
//    rfm69_test4();      // test OOK

// Initialize rfm69
    rfm69_init();       // OOK

//===========================================================
#if TESTLOOP
// Loop for testing
   while(1)
   {
      P1OUT |= BIT2;
      waitms(1);
      P1OUT &=~BIT2;
      waitms(1);
   }

#endif
//===========================================================

    device_id = i2c_read(0xD0);         // BME280 = 0x60, BMP280 = 0x58
    asm(" nop");    // for debugger, break here to get device ID

// Initialize bmp
    bme_init();
    getcal();      // get calibration data for temperature and humidity and make digs
    read_bme280();

    show_temperature = T+5000;   // for testing the sensor  7403 is 2430 is 24.3 degrees C is 75.7 degrees F
    show_humidity = Humidity;    // for testing the sensor  23 is 23%
    asm(" nop");    // for debugger, break here to check that the sensor is working

// This is used during receiver setup
    temperature_bcd4(&temp1, T+5000);  // adds 50 deg C for display  - makesit
    humidity_bcd(&hum1, Humidity);    // makes the transmitter data packet

// Select the sensor id
    if(P1IN & BIT3)         // ID select
    {
       makeid(&temp1,26);
       makeid(&hum1,26);
    }
    else
    {
       makeid(&temp1,27);
       makeid(&hum1,27);
    }

// Set up timer Interrupts
    CCTL0 = CCIE;                    // CCR0 interrupt enabled
    CCR0 = 31600;                    // 1 second interrupts (15800 = 500ms)
    TACTL = TASSEL_2 | ID_2 | MC_1;   // SMCLK, contmode = count up to CCR0
    TACTL |= TACLR;          // clear timer Resets TAR, clock divider, count direction

    __bis_SR_register(LPM0_bits + GIE);       // Timer interrupts with LPM0 (Sleep until the next interrupt)

// It should never get here
    while (1)           // main loop
    {
       P2OUT ^= BIT2;   // error
    }  // end while main loop

}  // end main
//==========================================================
// Timer A0 interrupt service routine
#if defined(__TI_COMPILER_VERSION__) || defined(__IAR_SYSTEMS_ICC__)
#pragma vector=TIMER0_A0_VECTOR
__interrupt void Timer_A (void)
#elif defined(__GNUC__)
void __attribute__ ((interrupt(TIMER0_A0_VECTOR))) Timer_A (void)
#else
#error Compiler not supported!
#endif
{

    // Reading the sensor has a 1 second interrupt all to itself
    if(irq_count == 2)
    {
       read_bme280();
       temperature_bcd4(&temp1, T+5000);
       humidity_bcd(&hum1, Humidity);
    }

    if(irq_count-- == 0)
    {

       switch (mode)
       {
          case 0:
            pwrup();                // power-up pulse 114ms, then wait 241ms
            sendit(&temp1, &hum1);  // 710 ms
            wait301ms();
            sendit(&temp1, &hum1);  // 710 ms
            mode = 1;               // do 7 more transmissions, 8 seconds apart
            count = 7;
            irq_count = 8;          // 8 seconds next time
            break;

          case 1:
            sendit(&temp1, &hum1);   // 710 ms
            if(count-- == 0)
            {
               mode = 2;    // when we are done with the first transmissions
            }
            irq_count = 8;  // 8-second transmission rate
            break;

          case 2:
            sendit(&temp1, &hum1);   // 710 ms
            irq_count = 59;          // 59 stay in mode 2 normally 56 seconds
            break;
       }  // end switch

    } // end irq counts

    asm(" nop");    // for debugger

 } // end irq
//==============================================
void waitms(unsigned int k)     // ms
{
   volatile unsigned int i,j;

   for(i=k; i>0; i--)
   {
      for(j=99; j>0; j--);
   }

   return;
}
//==============================================
void wait5ms(void)     // 5ms
{
   volatile unsigned int i,j;

   for(i=41; i>0; i--)
   {
      for(j=10; j>0; j--);
   }

   return;
}
//==========================================================
void wait29ms(void)     // 29ms
{
   volatile unsigned int i,j;

   for(i=241; i>0; i--)
   {
      for(j=10; j>0; j--);
   }

   return;
}
//==========================================================
void wait114ms(void)     // 114ms
{
   volatile unsigned int i,j;

   for(i=948; i>0; i--)
   {
      for(j=10; j>0; j--);
   }

   return;
}
//==========================================================
void wait195ms(void)    // 195 ms
{
   volatile unsigned int i,j;

   for(i=1610; i>0; i--)
   {
      for(j=10; j>0; j--);
   }

   return;
}
//==========================================================
void wait241ms(void)      // 241 ms
{
   volatile unsigned int i,j;

   for(i=1990; i>0; i--)
   {
      for(j=10; j>0; j--);
   }

   return;
}
//==========================================================
void wait301ms(void)     // 301 ms
{
   volatile unsigned int i,j;

   for(i=2499; i>0; i--)
   {
      for(j=10; j>0; j--);
   }

   return;
}
//==============================================
void waitseconds(unsigned int n)
{
    volatile unsigned int i,j,k;

    for(k=n; k>0; k--)
    {
       for(i=8200; i>0; i--)
       {
          for(j=10; j>0; j--);
       }
    }
    return;
}
//==============================================
void send_zero(void)    // transmitter
{
   unsigned int i;

   P1OUT |= BIT2;
   for(i=340; i>0; i--)    // 340 = 1360 us,350 = 1400us
   {
      asm(" nop");    // force it to compile
   }

   P1OUT &= ~BIT2;
   for(i=234; i>0; i--)    // 234 = 970 us, 241 = 1000
   {
      asm(" nop");    // force it to compile
   }
   return;
}
//==================================
void send_one(void)     // transmitter
{
    unsigned int i;

    P1OUT |= BIT2;
    for(i=143; i>0; i--)    // 143 = 580us, 136 = 550us550us
    {
       asm(" nop");    // force it to compile
    }

    P1OUT &= ~BIT2;
    for(i=234; i>0; i--)    // 234 = 970us, 241 = 1000us
    {
       asm(" nop");    // force it to compile
    }

    return;
}
//==============================================
// MSB first, '1' = Zero. '0' = One
void send_nybble(unsigned char data)
{
   unsigned int n;

   for(n = 4; n>0; n--)
   {
      if(data & 0x08)
      {
         send_one();
      }
      else
      {
         send_zero();
      }

      data = data<<1;
   }
}
//===========================================
// MSB first, '1' = Zero. '0' = One
void send_byte(unsigned char data)
{
   unsigned int n;

   for(n = 8; n>0; n--)
   {
      if(data & 0x80)
      {
         send_one();
      }
      else
      {
         send_zero();
      }

      data = data<<1;
   }
}
//===========================================
void send_word(unsigned int data)   // MSB first, '1' = Zero. '0' = One
{
   unsigned int n;

   for(n = 16; n>0; n--)
   {
      if(data & 0x8000)
      {
         send_one();
      }
      else
      {
         send_zero();
      }

      data = data<<1;
   }
}
// ==========================================
void makeid(struct sensor *sptr, unsigned char id)
{
    sptr->address = id;
}
//==========================================
void makeit(struct sensor *sptr)     // make the data packet for the transmitter
{
    unsigned char parity, temp;
    unsigned int n;

// sptr points to the sensor data block

// Make sure data4 and data3 are duplicated
   sptr->data1 = sptr->data4;
   sptr->data0 = sptr->data3;

// Make parity only from the three nybbles of the data. Ignore the address and the data repeats
    parity = 0;

    temp = sptr->data4;
    for(n=4; n>0; n--)
    {
      if (temp & 0x01)
      { parity++; }
      temp = temp>>1;
    }

    temp = sptr->data3;
    for(n=4; n>0; n--)
    {
      if (temp & 0x01)
      {parity++;}
      temp = temp>>1;
    }

    temp = sptr->data2;
    for(n=4; n>0; n--)
    {
      if (temp & 0x01)
      {parity++;}
      temp = temp>>1;
    }

// we have parity
    sptr->daddress = (sptr->address<<1) | (parity & 0x01);

    sptr->checksum = 0x0A + sptr->type + (sptr->daddress>>4) + (sptr->daddress&0x0F) + sptr->data4 + sptr->data3 + sptr->data2 + sptr->data1 + sptr->data0;
    sptr->checksum = sptr->checksum & 0x0F;
/*
// For testing
    temp = sptr->type;
    temp = sptr->daddress;
    temp = sptr->data4;
    temp = sptr->data3;
    temp = sptr->data2;
    temp = sptr->data1;
    temp = sptr->data0;
    temp = sptr->checksum;

    asm(" nop");    // for emulator
*/
    return;
}
//==================================
void sendit(struct sensor *sptr1, struct sensor *sptr2)
{
// wake up
   spi_write(RegOpMode,0x0C);      // Transmitter mode (TX)

// send first packet (temperature)
    send_packet(sptr1);

// wait 29ms and repeat the temperature data packet
    wait29ms();
    send_packet(sptr1);

// wait 301ms to send humidity packet
    wait301ms();

// send humidity packet    0x0AEF, 0xE0FF, 0x00F2};   humidity
    send_packet(sptr2);

// wait 29ms and repeat the humidity packet
    wait29ms();        // wait and repeat data packet
    send_packet(sptr2);

// Back to sleep
    spi_write(RegOpMode,0x00);      // Sleep mode
}
//==========================================
void send_packet(struct sensor *sptr)
{
    send_byte(0x0A);
    send_nybble(sptr->type);
    send_nybble( (sptr->daddress>>4) );
    send_nybble( (sptr->daddress & 0x0F) );
    send_nybble(sptr->data4);
    send_nybble(sptr->data3);
    send_nybble(sptr->data2);
    send_nybble(sptr->data1);
    send_nybble(sptr->data0);
    send_nybble(sptr->checksum);
}
//=========================================
// power-up pulse 114ms
void pwrup()
{
    P1OUT |= BIT2;
    wait114ms();
    P1OUT &= ~BIT2;

// wait 241ms
    wait241ms();
}
//===========================================
// Convert Binary temperature to BCD - 3 digits; was used for testing
void temperature_bcd(struct sensor *sptr, unsigned int bin)  // example: temperature_bcd(23,&temp1);
{
   unsigned char bcd0, bcd1, bcd2;
   int i;
   unsigned char hcarry;

// Adapted from 8-bit microcontroller code using a DAA instruction
// bin = 39
   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;

   for(i=0; i<12; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x800)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
//       bcd3 = bcd3 + 1;
//       hcarry = 1;
     }
//        else {hcarry = 0;}

   }  // end for

   sptr->data4 = bcd2;
   sptr->data3 = bcd1;
   sptr->data2 = bcd0;

   makeit(sptr);  // make the data packet

   asm(" nop");    // for emulator
}
//=========================================
// Convert Binary temperature to BCD - 4 digits from BMP sensor
void temperature_bcd4(struct sensor *sptr, unsigned int bin)  // example: temperature_bcd(23,&temp1);
{
   unsigned char bcd0, bcd1, bcd2, bcd3;
   int i;
   unsigned char hcarry;

//   bin = 1234;   // test

// Adapted from 8-bit microcontroller code using a DAA instruction
// bin = 39
   bcd0 = 0;
   bcd1 = 0;
   bcd2 = 0;
   bcd3 = 0;

   for(i=0; i<16; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x8000)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

//---------- BCD2
      bcd2 = bcd2<<1;
      bcd2 = bcd2 + hcarry;

     if(bcd2 >9)
     {
         bcd2 = bcd2 - 10;
         hcarry = 1;
     }
        else {hcarry = 0;}

//---------- BCD3
     bcd3 = bcd3<<1;
     bcd3 = bcd3 + hcarry;

     if(bcd3 >9)
     {
         bcd3 = bcd3 - 10;
//       hcarry = 1;
     }
//        else {hcarry = 0;}

   }  // end for

   sptr->data4 = bcd3;
   sptr->data3 = bcd2;
   sptr->data2 = bcd1;      // bcd0 is not used

   makeit(sptr);  // make the data packet

   asm(" nop");    // for emulator
}
//=========================================
// Convert Binary humidity to BCD
void humidity_bcd(struct sensor *sptr, unsigned int bin)  // example: humidity_bcd(23,&hum1);
{
   unsigned char bcd0, bcd1;
   int i;
   unsigned char hcarry;

   if(bin>99)  bin=99;   // maximum number

// Adapted from 8-bit microcontroller code using a DAA instruction
// bin = 39
   bcd0 = 0;
   bcd1 = 0;

   for(i=0; i<8; i++)
   {
//---------- BCD0
      bcd0 = bcd0<<1;

      if(bin & 0x80)
      {bcd0 = bcd0 +1;}

      bin = bin<<1;

      if(bcd0 >9)
      {
         bcd0 = bcd0 - 10;
         hcarry = 1;
      }
      else {hcarry = 0; }

//---------- BCD1
      bcd1 = bcd1<<1;
      bcd1 = bcd1 + hcarry;

      if(bcd1 >9)
      {
          bcd1 = bcd1 - 10;
          hcarry = 1;
      }
      else {hcarry = 0;}

   }  // end for

   sptr->data4 = bcd1;
   sptr->data3 = bcd0;
   sptr->data2 = 0;

   makeit(sptr);  // make the data packet

   asm(" nop");    // for emulator
}
//=========================================
void wait1ms()
{
    volatile unsigned int i;

    for(i=89; i>0; i--)
    {
        asm(" nop");    // for debugger
    }
    return;
}
//========================================
