//***************************************************************************************
// RFM69 Transceiver
// Do SPI bit banger
// Jed Margolin  2/1/2021.
//***************************************************************************************

void wait1ms(void);
void wait1(void);

void spi_write(unsigned char, unsigned char);
unsigned char spi_read(unsigned char);

void rfm69_init(void);
int wait_rfm_ready(void);
void wait_rfm1(void);

unsigned char spi_write_flag;
unsigned char spi_read_flag;

unsigned char rfmdata[8];
void read_rfm(void);
void rfm_test(void);
//================================================
/* Transmitter Test

    spi_write(RegOpMode,0x0C);  // Transmitter mode (TX)
    spi_write(RegDataModul,0x00);   // Packet Mode, FSK, No Modulation Shaping

    spi_write(RegFdevMsb,0x00);     // MSB of the frequency deviation: [5 KHz]
    spi_write(RegFdevLsb,0x52);         // LSB of the frequency deviation

    spi_write(RegFrfMsb,0x6C);  // MSB of the RF carrier frequency   433.92MHz
    spi_write(RegFrfMid,0x7A);      // Middle Byte of the RF carrier frequency
    spi_write(RegFrfLsb,0xE1);      // LSB of the RF carrier frequency

    spi_write(RegPaLevel,0x9F); // Enables PA0, connected to RFIO and LNA; Output power setting,
//                      with 1 dB steps Pout = -18 + OutputPower [dBm] , with PA0

//  spi_write(RegDioMapping1,0xx);  // work on this

    flags = spi_read(RegIrqFlags1);

    spi_write(RegPreambleMsb,0);    // Size of the preamble to be sent (from TxStartCondition fulfilled). (MSB byte)
    spi_write(RegPreambleLsb,3);    // Size of the preamble to be sent (from TxStartCondition fulfilled). (LSB byte)
    spi_write(RegSyncConfig,0x99);  // Sync word generation; Sync word size = 2

    spi_write(RegSyncValue1,0x01;   // 1st byte of Sync word
    spi_write(RegSyncValue2,0x02;       // 2nd byte of Sync word

    spi_write(RegPacketConfig1,0x00);      // Vanilla
    spi_write(RegPacketConfig1,0x50);      // Fixed Length; Whitening; Enable CRC

    spi_write(RegPayloadLength,16); // payload length

    spi_write(NodeAddress,0x55);    // Node address used in address filtering

    spi_write(RegBroadcastAdrs,0x33);      // Broadcast address used in address filtering

    spi_write(RegFifoThresh,0x00);  // Defines the condition to start packet transmission; the number of bytes in the FIFO exceeds FifoThreshold

//  spi_write(RegFifoThresh,0x00);  // Defines the condition to start packet transmission; the number of bytes in the FIFO exceeds FifoThreshold
    spi_write(RegFifoThresh,0x80);  // Defines the condition to start packet transmission; at least one byte in the FIFO

    data = 0x00;
    while(1)
    {
         wait1ms();
         for(n = 16; n>0; n--)
        {
            spi_write(Fifo, data++);
       }
           {
end transmitter test
*/

// RFM69 Registers

// Table 24 Common Configuration Registers

//   Name(Address)   Bits  Variable Name   Mode  Default Value  Description
//
#define RegFifo (0x00)
//  7-0     Fifo            rw  0x00            FIFO data input


// RegOpMode = 0x0C;        // Transmitter mode (TX)
#define RegOpMode (0x01)
/*
    7   SequencerOff            rw  0   Controls the automatic Sequencer (see section 4.2 ):

        0 ? Operating mode as selected with Mode bits in RegOpMode is automatically reached with the Sequencer
        1 ? Mode is forced by the user

    6   ListenOn            rw  0   Enables Listen mode, should be enabled whilst in Standby mode:

        0 ? Off (see section 4.3)
        1 ? On

    5   ListenAbort             w   0   Aborts Listen mode when set together with ListenOn=0
                                See section 4.3.4 for details
                                Always reads 0.

    4-2     Mode                rw  001     Transceivers operating modes:

        000 ? Sleep mode (SLEEP)
        001 ? Standby mode (STDBY)
        010 ? Frequency Synthesizer mode (FS)
        011 ? Transmitter mode (TX)
        100 ? Receiver mode (RX)
        others ? reserved
        Reads the value corresponding to the current chip mode

    1-0                 r   00      unused
*/

// RegDataModul = 0x00; // Packet Mode, FSK, No Modulation Shaping
#define RegDataModul (0x02)
/*
    6-5     DataMode            rw  00      Data processing mode:

        00 ? Packet mode
        01 ? reserved
        10 ? Continuous mode with bit synchronizer
        11 ? Continuous mode without bit synchronizer

    4-3     ModulationType      rw  00      Modulation scheme:

        00 ? FSK
        01 ? OOK
        10 - 11 ? reserved
    2                   r   0       unused

    1-0     ModulationShaping       rw  00      Data shaping:

    in FSK:
        00 ? no shaping
        01 ? Gaussian filter, BT = 1.0
        10 ? Gaussian filter, BT = 0.5
        11 ? Gaussian filter, BT = 0.3

    in OOK:
        00 ? no shaping
        01 ? filtering with fcutoff = BR
        10 ? filtering with fcutoff = 2*BR
        11 ? reserved
*/

// RegBitrateMsb = 0x1A;    // MSB of Bit Rate when Manchester encoding is used [4.8 kb/s]
#define RegBitrateMsb (0x03)
//  7-0     BitRate(15:8)       rw  0x1a        MSB of Bit Rate (Chip Rate when Manchester encoding is enabled)

// RegBitrateLsb = 0x0B;    // LSB of Bit Rate when Manchester encoding is used
#define RegBitrateLsb (0x04)
/*
    7-0     BitRate(7:0)        rw  0x0b    LSB of Bit Rate (Chip Rate if Manchester encoding is enabled)

            BitRate = FXO SC /BitRate(15,0) Default value: 4.8 kb/s

    [Default value: 4.8 kb/s]


// RegFdevMsb = 0x00;   // MSB of the frequency deviation: [5 KHz]
// RegFdevLsb = 0x52;     // LSB of the frequency deviation
*/

#define RegFdevMsb (0x05)
/*
    7-6                 r   00  unused

    5-0     Fdev(13:8)      rw  000000  MSB of the frequency deviation
*/

#define RegFdevLsb (0x06)
//  7-0 Fdev(7:0)           rw  0x52        LSB of the frequency deviation; Fdev = Fstep ? Fdev(15,0)
//  [Default value: 5 kHz]


// RegFrfMsb = 0x6C     ; MSB of the RF carrier frequency    433.92MHz
// RegFrfMid =  0x7A    ; Middle Byte of the RF carrier frequency
// RegFrfLsb =  0xE1    ; LSB of the RF carrier frequency
#define RegFrfMsb (0x07)
//  7-0     Frf(23:16)      rw  0xe4        MSB of the RF carrier frequency


#define RegFrfMid (0x08)
//  7-0     Frf(15:8)       rw  0xc0        Middle byte of the RF carrier frequency

#define RegFrfLsb (0x09)
/*
    7-0     Frf(7:0)        rw  0x00        LSB of the RF carrier frequency

    Frf = Fstep ? Frf (23;0)?
    Default value: Frf = 915 MHz (32 MHz XO)    fstep for 32MHz xtal  = 32MHz/(2^19) = 61.03515625Hz
    freq = 61.03515625Hz  * 0xE4C000 = 61.03515625Hz * 14991360 = 915.000 MHz
    For 433.92MHz, scale = 433.92MHz/ 61.03515625 = 7,109,345d =  6C7AE1 = 6C 7A E1


    7   RcCalStart      w   0       Triggers the calibration of the RC oscillator when set.
                                Always reads 0. RC calibration must be triggered in Standby mode.

    6   RcCalDone       r   1       0 ? RC calibration in progress,     1 ? RC calibration is over

    5-0                 r    000001     unused
*/


#define RegOsc1 (0x0A)
/*
    7-6             r    00         unused

    5   AfcLowBetaOn    rw  0       Improved AFC routine for signals with modulation index lower than 2.
                                Refer to section 3.4.16 for details
        0 ? Standard AFC routine
        1 ? Improved AFC routine
*/

#define RegAfcCtrl (0x0B)
//  4-0                 r   00000       unused

#define Reserved0C (0x0C)
//  7-0                 r   0x02        unused

#define RegListen1 (0x0D)
/*
    7-6     ListenResolIdle     rw  10  Resolution of Listen mode Idle time (calibrated RC osc):
        00 ? reserved
        01 ? 64 us
        10 ? 4.1 ms
        11 ? 262 ms

    5-4     ListenResolRx   rw  01  Resolution of Listen mode Rx time (calibrated RC osc):
        00 ? reserved
        01 ? 64 us
        10 ? 4.1 ms
        11 ? 262 ms

    3   ListenCriteria  rw  0   Criteria for packet acceptance in Listen mode:

        0 ? signal strength is above RssiThreshold
        1 ? signal strength is above RssiThreshold and SyncAddress matched

    2-1     ListenEnd       rw  01  Action taken after acceptance of a packet in Listen mode:
        00 ? chip stays in Rx mode. Listen mode stops and must be disabled (see section 4.3).
        01 ? chip stays in Rx mode until PayloadReady or Timeout interrupt occurs. It then goes to the mode defined
            by Mode. Listen mode stops and must be disabled (see section 4.3).
        10 ? chip stays in Rx mode until PayloadReady orTimeout interrupt occurs. Listen mode then resumes in
            Idle state. FIFO content is lost at next Rx wakeup.
        11 ? Reserved
    0               r   0   unused
*/

#define RegListen2 (0x0E)
//  7-0     ListenCoefIdle  rw  0xf5    Duration of the Idle phase in Listen mode.
//                          tListenIdle = ListenCoefIdle ? ListenResolIdle

#define RegListen3 (0x0F)
//  7-0     ListenCoefRx        rw  0x20    Duration of the Rx phase in Listen mode (startup time included, see section 4.2.3)
// tListenRx = ListenCoefRx ? ListenResolRx

#define RegVersion (0x10)
//  7-0     Version         r   0x24    Version code of the chip. Bits 7-4 give the full revision number; bits 3-0 give the metal mask revision                                 number.
//=================================

// Table 25 Transmitter Registers

// RegPaLevel = 0x9F;   // Enables PA0, connected to RFIO and LNA; Output power setting, with 1 dB steps Pout = -18 + OutputPower [dBm] , with PA0
#define RegPaLevel (0x11)
/*
    7   Pa0On *         rw  1       Enables PA0, connected to RFIO and LNA
    6   Pa1On *         rw  0       Enables PA1, on PA_BOOST pin
    5   Pa2On *         rw  0       Enables PA2, on PA_BOOST pin
    4-0     OutputPower         rw  11111       Output power setting, with 1 dB steps
                                Pout = -18 + OutputPower [dBm] , with PA0
*/

#define RegPaRamp (0x12)
/*
    7-4                 r   0000    unused
    3-0     PaRamp      rw  1001    Rise/Fall time of ramp up/down in FSK
        0000 ? 3.4 ms
        0001 ? 2 ms
        0010 ? 1 ms
        0011 ? 500 us
        0100 ? 250 us
        0101 ? 125 us
        0110 ? 100 us
        0111 ? 62 us
        1000 ? 50 us
        1001 ? 40 us
        1010 ? 31 us
        1011 ? 25 us
        1100 ? 20 us
        1101 ? 15 us
        1110 ? 12 us
        1111 ? 10 us
*/

#define RegOcp (0x13)
/*
    7-5             r   000     unused

    4   OcpOn   rw  1   Enables overload current protection (OCP) for the PA:
        0 ? OCP disabled
        1 ? OCP enabled

    3-0     OcpTrim     rw  1010    Trimming of OCP current:
                        Imax = 45 + 5 ? OcpTrim mA 95 mA OCP by default
*/

//=================================

//Table 26 - Receiver Registers

// Name (Address)   Bits    Variable Name Mode Default  Description Value

#define Reserved14 (0x14)
//  7-0             r   0x40        unused

#define Reserved15 (0x15)
//  7-0             r   0xB0        unused

#define Reserved16 (0x16)
//  7-0             r   0x7B        unused

#define Reserved17 (0x17)
//  7-0             r   0x9B        unused

#define RegLna  (0x18)
/*
    7   LnaZin      rw  1       LNAs input impedance
        0 . 50 ohms
        1 . 200 ohms

    6               r   0       unused

    5-3     LnaCurrentGain  r   001         Current LNA gain, set either manually, or by the AGC

    2-0     LnaGainSelect   rw  000     LNA gain setting:
        000 . gain set by the internal AGC loop
        001 . G1 = highest gain
        010 . G2 = highest gain  6 dB
        011 . G3 = highest gain  12 dB
        100 . G4 = highest gain  24 dB
        101 . G5 = highest gain  36 dB
        110 . G6 = highest gain  48 dB
        111 . reserved
*/

#define RegRxBw (0x19)
/*

    7-5     DccFreq         rw  010 * Cut-off frequency of the DC offset canceller (DCC): Formula on RFM69 Datasheet PDF page 67
                        ~4% of the RxBw by default

    4-3     RxBwMant        rw  10  * Channel filter bandwidth control:
        00 . RxBwMant = 16      10 . RxBwMant = 24
        01 . RxBwMant = 20      11 . reserved

    2-0     RxBwExp         rw  101     * Channel filter bandwidth control:

    FSK Mode: PDF Formula on RFM69 Datasheet PDF page 67

    OOK Mode: Formula on RFM69 Datasheet PDF page 67

    See RFM69 Datasheet Table 14 for tabulated values
*/

#define RegAfcBw (0x1A)     // default 0x8B
/*
    7-5     DccFreqAfc      rw  100     DccFreq parameter used during the AFC

    4-3     RxBwMantAfc     rw  01  RxBwMant parameter used during the AFC

    2-0     RxBwExpAfc      rw  011     * RxBwExp parameter used during the AFC
*/

#define RegOokPeak (0x1B)
/*
    7-6     OokThreshType   rw  01  Selects type of threshold in the OOK data slicer:
        00 . fixed 10 . average
        01 . peak 11 . reserved

    5-3     OokPeakTheshStep    rw  000     Size of each decrement of the RSSI threshold in the OOK demodulator:
        000 . 0.5 dB 001 . 1.0 dB
        010 . 1.5 dB 011 . 2.0 dB
        100 . 3.0 dB 101 . 4.0 dB
        110 . 5.0 dB 111 . 6.0 dB

    2-0     OokPeakThreshDec        rw  000     Period of decrement of the RSSI threshold in the OOK demodulator:
        000 . once per chip 001 . once every 2 chips
        010 . once every 4 chips 011 . once every 8 chips
        100 . twice in each chip 101 . 4 times in each chip
        110 . 8 times in each chip 111 . 16 times in each chip
*/

#define RegOokAvg (0x1C)
/*
    7-6     OokAverageThreshFilt    rw  10  Filter coefficients in average mode of the OOK demodulator:
        00 . fC  chip rate / 32.p 01 . fC  chip rate / 8.p
        10 . fC  chip rate / 4.p 11 .fC  chip rate / 2.p

    5-0                     r   000000 unused
*/

#define RegOokFix (0x1D)
//  7-0     OokFixedThresh      rw  0110    (6dB) Fixed threshold value (in dB) in the OOK demodulator. Used when OokThresType = 00

#define RegAfcFei (0x1E)
/*
    7               r    0  unused

    6   FeiDone         r    0  0 .FEI is on-going; 1 . FEI finished

    5   FeiStart        w   0   Triggers a FEI measurement when set. Always reads 0.

    4   AfcDone         r   1   0 . AFC is on-going; 1 . AFC has finished

    3   AfcAutoclearOn  rw  0   Only valid if AfcAutoOn is set
            0 . AFC register is not cleared before a new AFC phase
            1 . AFC register is cleared before a new AFC phase

    2   AfcAutoOn       rw  0   0 . AFC is performed each time AfcStart is set; 1 . AFC is performed each time Rx mode is entered

    1   AfcClear        w   0   Clears the AfcValue if set in Rx mode. Always reads 0

    0   AfcStart        w   0   Triggers an AFC when set. Always reads 0.
*/

#define RegAfcMsb (0x1F)
//  7-0 AfcValue(15:8)  r   0x00    MSB of the AfcValue, 2s complement format

#define RegAfcLsb (0x20)
//  7-0 AfcValue(7:0)   r   0x00    LSB of the AfcValue, 2s complement format Frequency correction = AfcValue x Fstep

#define RegFeiMsb (0x21)
//  7-0 FeiValue(15:8)  r   MSB of the measured frequency offset, 2s complement

#define RegFeiLsb (0x22)
//  7-0 FeiValue(7:0)   r   LSB of the measured frequency offset, 2s complement
//                    Frequency error = FeiValue x Fstep

#define RegRssiConfig (0x23)
/*
    7-2             r       000000 unused

    1   RssiDone    r   1   0 . RSSI is on-going
                        1 . RSSI sampling is finished, result available
                        0 RssiStart w 0 Trigger a RSSI measurement when set. Always reads 0.
*/

#define RegRssiValue (0x24)
//  7-0     RssiValue   r   0xFF A  bsolute value of the RSSI in dBm, 0.5dB steps.
//                            RSSI = -RssiValue/2 [dBm]

//======================================

// Table 27 - IRQ and Pin Mapping Registers

//Name (Address)  Bits  Variable Name  Mode  Default  Description  Value

//RegDioMapping1 = xxx;
#define RegDioMapping1 (0x25)
/*
    7-6     Dio0Mapping         rw  00      Mapping of pins DIO0 to DIO5
                                See Table 21 for mapping in Continuous mode
                                See Table 22 for mapping in Packet mode
    5-4     Dio1Mapping         rw  00
    3-2     Dio2Mapping         rw  00
    1-0     Dio3Mapping         rw  00
*/

#define RegDioMapping2 (0x26)
/*
    7-6

    5-4     Dio5Mapping         rw  00

    3               r   0   unused

    2-0     ClkOut      rw  111     * Selects CLKOUT frequency:
        000 . FXOSC
        001 . FXOSC / 2
        010 . FXOSC / 4
        011 . FXOSC / 8
        100 . FXOSC / 16
        101 . FXOSC / 32
        110 . RC (automatically enabled)
        111 . OFF
*/
// flags = spi_read(RegIrqFlags1);
#define RegIrqFlags1 (0x27)
/*
    7   ModeReady   r   1   Set when the operation mode requested in Mode, is ready
        - Sleep: Entering Sleep mode
        - Standby: XO is running
        - FS: PLL is locked
        - Rx: RSSI sampling starts
        - Tx: PA ramp-up completed
    Cleared when changing operating mode.

    6   RxReady     r   0   Set in Rx mode, after RSSI, AGC and AFC.
                        Cleared when leaving Rx.

    5   TxReady     r   0   Set in Tx mode, after PA ramp-up.
                        Cleared when leaving Tx.

    4   PllLock     r   0   Set (in FS, Rx or Tx) when the PLL is locked.
                        Cleared when it is not.

    3   Rssi        rwc     0   Set in Rx when the RssiValue exceeds RssiThreshold.
                        Cleared when leaving Rx.

    2   Timeout  r  0   Set when a timeout occurs (see TimeoutRxStart and TimeoutRssiThresh)
                        Cleared when leaving Rx or FIFO is emptied.

    1   AutoMode    r   0   Set when entering Intermediate mode.
                        Cleared when exiting Intermediate mode.
Please note that in Sleep mode a small delay can be observed between AutoMode interrupt and the corresponding enter/exit condition.

    0   SyncAddressMatch    r/rwc   0   Set when Sync and Address (if enabled) are detected.
                            Cleared when leaving Rx or FIFO is emptied.
                            This bit is read only in Packet mode, rwc in Continuous mode
*/

#define RegIrqFlags2 (0x28)
/*
    7   FifoFull    r   0   Set when FIFO is full (i.e. contains 66 bytes), else cleared.

    6   FifoNotEmpty r 0 Set when FIFO contains at least one byte, else cleared

    5   FifoLevel   r   0   Set when the number of bytes in the FIFO strictly exceeds FifoThreshold, else cleared.

    4   FifoOverrun     rwc     0   Set when FIFO overrun occurs. (except in Sleep mode) Flag(s) and FIFO are cleared when this bit is set.
                        The FIFO then becomes immediately available for the next transmission / reception.
    3   PacketSent  r   0   Set in Tx when the complete packet has been sent. Cleared when exiting Tx.

    2   PayloadReady r  0   Set in Rx when the payload is ready (i.e. last byte received and CRC, if enabled and CrcAutoClearOff is cleared, is Ok).
                        Cleared when FIFO is empty.

    1   CrcOk       r   0   Set in Rx when the CRC of the payload is Ok. Cleared when FIFO is empty. 0 -r 0 unused

*/

#define RegRssiThresh (0x29)
//  7-0     RssiThreshold   rw  0xE4    *RSSI trigger level for Rssi interrupt:  RssiThreshold / 2 [dBm]

#define RegRxTimeout1 (0x2A)
//  7-0     TimeoutRxStart   rw     0x00    Timeout interrupt is generated TimeoutRxStart*16*Tbit after switching to Rx mode if Rssi interrupt doesnt occur
//                      (i.e. RssiValue > RssiThreshold); 0x00: TimeoutRxStart is disabled

#define RegRxTimeout2 (0x2B)
//  7-0     TimeoutRssiThresh rw    0x00    Timeout interrupt is generated TimeoutRssiThresh*16*Tbit after Rssi interrupt if
//                          PayloadReady interrupt doesnt occur. 0x00: TimeoutRssiThresh is disabled

//===========================================

// Table 28 Packet Engine Registers

// Name (Address)  Bits  Variable Name  Mode  Default  Description  Value

// PreambleSize
// RegPreambleMsb = 0;
#define RegPreambleMsb (0x2c)
//  7-0     PreambleSize(15:8)  rw  0x00    Size of the preamble to be sent (from TxStartCondition fulfilled). (MSB byte)

//RegPreambleLsb = 3;
#define RegPreambleLsb (0x2d)
//  7-0     PreambleSize(7:0)   rw  0x03    Size of the preamble to be sent (from TxStartCondition fulfilled). (LSB byte)

// RegSyncConfig = 0x99;        // Sync word generation; Sync word size = 2
#define RegSyncConfig (0x2e)
/*
    7   SyncOn      rw  1   Enables the Sync word generation and detection:
        0 = Off
        1 = On(0x2e)

    6   FifoFillCondition   rw  0   FIFO filling condition:
        0 = if SyncAddress interrupt occurs
        1 = as long as FifoFillCondition is set

    5-3     SyncSize        rw  011     Size of the Sync word: (SyncSize + 1) bytes

    2-0     SyncTol         rw  000     Number of tolerated bit errors in Sync word
*/

// RegSyncValue1 = 0x01;    // 1st byte of Sync word
// RegSyncValue2 = 0x02;   // 2nd byte of Sync word
#define RegSyncValue1 (0x2f)
//  7-0     SyncValue(63:56)    rw  0x01    *1st byte of Sync word. (MSB byte) Used if SyncOn is set.

#define RegSyncValue2 (0x30)
//  7-0     SyncValue(55:48)    rw  0x01    *2nd byte of Sync word Used if SyncOn is set and (SyncSize +1) >= 2. (0x30)

#define RegSyncValue3 (0x31)
//  7-0     SyncValue(47:40)    rw  0x01    *3rd byte of Sync word. Used if SyncOn is set and (SyncSize +1) >= 3. (0x31)

#define RegSyncValue4 (0x32)
//  7-0     SyncValue(39:32)    rw  0x01    *4th byte of Sync word. Used if SyncOn is set and (SyncSize +1) >= 4. (0x32)

#define RegSyncValue5 (0x33)
//  7-0     SyncValue(31:24)    rw  0x01    *5th byte of Sync word. Used if SyncOn is set and (SyncSize +1) >= 5. (0x33)

#define RegSyncValue6 (0x34)
//  7-0     SyncValue(23:16)    rw  0x01    *6th byte of Sync word. Used if SyncOn is set and (SyncSize +1) >= 6. (0x34)

#define RegSyncValue7 (0x35)
//  7-0     SyncValue(15:8)     rw  0x01    *7th byte of Sync word. Used if SyncOn is set and (SyncSize +1) >= 7. (0x35)

#define RegSyncValue8 (0x36)
//  7-0     SyncValue(7:0)  rw  0x01    *8th byte of Sync word. Used if SyncOn is set and (SyncSize +1) = 8.(0x36)

// RegPacketConfig1 = 0x00      // Vanilla
// RegPacketConfig1 = 0x50      // Fixed Length; Whitening; Enable CRC
#define RegPacketConfig1 (0x37)
/*
    7   PacketFormat        rw  0   Defines the packet format used:
                            0 . Fixed length
                            1 . Variable length

    6-5     DcFree      rw  00  Defines DC-free encoding/decoding performed:
            00 . None (Off)
            01 . Manchester
            10 . Whitening
            11 . reserved

    4   CrcOn           rw  1   Enables CRC calculation/check (Tx/Rx):
            0 . Off
            1 . On

    3   CrcAutoClearOff     rw  0   Defines the behavior of the packet handler when CRC check fails:
            0 . Clear FIFO and restart new packet reception. No PayloadReady interrupt issued.
            1 . Do not clear FIFO. PayloadReady interrupt issued.

    2-1     AddressFiltering    rw  00  Defines address based filtering in Rx:
            00 . None (Off)
            01 . Address field must match NodeAddress
            10 . Address field must match NodeAddress or BroadcastAddress
            11 . reserved

    0               rw  0   unused
*/

// RegPayloadLength = 16;       // payload length
#define RegPayloadLength (0x38)
//  7-0     PayloadLength   rw  0x40    If PacketFormat = 0 (fixed), payload length.
//                            If PacketFormat = 1 (variable), max length in Rx, not used in Tx.
// NodeAddress = 0x55;      // Node address used in address filtering

#define RegNodeAdrs (0x39)
//  7-0     NodeAddress         rw  0x00    Node address used in address filtering.

// RegBroadcastAdrs = 0x33;     // Broadcast address used in address filtering
#define RegBroadcastAdrs (0x3A)
//  7-0     BroadcastAddress    rw  0x00    Broadcast address used in address filtering.

#define RegAutoModes (0x3B)
/*
    7-5     EnterCondition  rw  000     Interrupt condition for entering the intermediate mode:
        000 . None (AutoModes Off)
        001 . Rising edge of FifoNotEmpty
        010 . Rising edge of FifoLevel
        011 . Rising edge of CrcOk
        100 . Rising edge of PayloadReady
        101 . Rising edge of SyncAddress
        110 . Rising edge of PacketSent
        111 . Falling edge of FifoNotEmpty (i.e. FIFO empty)

    4-2 ExitCondition   rw  000     Interrupt condition for exiting the intermediate mode:
        000 . None (AutoModes Off)
        001 . Falling edge of FifoNotEmpty (i.e. FIFO empty)
        010 . Rising edge of FifoLevel or Timeout
        011 . Rising edge of CrcOk or Timeout
        100 . Rising edge of PayloadReady or Timeout
        101 . Rising edge of SyncAddress or Timeout
        110 . Rising edge of PacketSent
        111 . Rising edge of Timeout

    1-0     IntermediateMode rw 00 Intermediate mode:
        00 . Sleep mode (SLEEP)
        01 . Standby mode (STDBY)
        10 . Receiver mode (RX)
        11 . Transmitter mode (TX)
*/

// RegFifoThresh = 0x00;    // Defines the condition to start packet transmission; the number of bytes in the FIFO exceeds FifoThreshold
// RegFifoThresh = 0x80;    // Defines the condition to start packet transmission; at least one byte in the FIFO
#define RegFifoThresh (0x3C)
/*
    7   TxStartCondition    rw  1   *Defines the condition to start packet transmission :
        0 . FifoLevel (i.e. the number of bytes in the FIFO exceeds FifoThreshold)
        1 . FifoNotEmpty (i.e. at least one byte in the FIFO)

    6-0     FifoThreshold   rw  0001111     Used to trigger FifoLevel interrupt.
*/

#define RegPacketConfig2 (0x3D)
/*
    7-4     InterPacketRxDelay  rw  0000    After PayloadReady occurred, defines the delay between FIFO empty and the start
                            of a new RSSI phase for next packet. Must match the transmitters PA ramp-down time.
            - Tdelay = 0 if InterpacketRxDelay >= 12

            - Tdelay = (2InterpacketRxDelay) / BitRate otherwise

    3               rw  0   unused

    2   RestartRx       w   0   Forces the Receiver in WAIT mode, in Continuous Rx mode. Always reads 0.

    1   AutoRxRestartOn     rw  1   Enables automatic Rx restart (RSSI phase) after PayloadReady occurred and packet
                            has been completely read from FIFO:
                                0 . Off. RestartRx can be used.
                                1 . On. Rx automatically restarted after InterPacketRxDelay.

    0   AesOn       rw  0   Enable the AES encryption/decryption: 0 . Off; 1 . On (payload limited to 66 bytes maximum)
*/

#define RegAesKey1 (0x3E)
//  7-0     AesKey(127:120)     w   0x00    1st byte of cipher key (MSB byte)

#define RegAesKey2 (0x3F)
//  7-0     AesKey(119:112)     w   0x00    2nd byte of cipher key

#define RegAesKey3 (0x40)
//  7-0     AesKey(111:104)     w   0x00    3rd byte of cipher key

#define RegAesKey4 (0x41)
//  7-0     AesKey(103:96)  w   0x00    4th byte of cipher key

#define RegAesKey5 (0x42)
//  7-0     AesKey(95:88)   w   0x00    5th byte of cipher key

#define RegAesKey6 (0x43)
//  7-0     AesKey(87:80)   w   0x00    6th byte of cipher key

#define RegAesKey7 (0x44)
//  7-0     AesKey(79:72)   w   0x00    7th byte of cipher key

#define RegAesKey8 (0x45)
//  7-0     AesKey(71:64)   w   0x00    8th byte of cipher key

#define RegAesKey9 (0x46)
//  7-0     AesKey(63:56)   w   0x00    9th byte of cipher key

#define RegAesKey10 (0x47)
//  7-0     AesKey(55:48)   w   0x00    10th byte of cipher key

#define RegAesKey11 (0x48)
//  7-0     AesKey(47:40)   w   0x00    11th byte of cipher key

#define RegAesKey12 (0x49)
//  7-0     AesKey(39:32)   w   0x00    12th byte of cipher key

#define RegAesKey13 (0x4A)
//  7-0     AesKey(31:24)   w   0x00    13th byte of cipher key

#define RegAesKey14 (0x4B)
//  7-0     AesKey(23:16)   w   0x00    14th byte of cipher key

#define RegAesKey15 (0x4C)
//  7-0     AesKey(15:8)    w   0x00    15th byte of cipher key

#define RegAesKey16 (0x4D)
//  7-0 AesKey(7:0)         w   0x00 16th byte of cipher key (LSB byte)

//===============================

// Table 29 - Temperature Sensor Registers

// Name (Address)  Bits  Variable Name  Mode  Default  Description  Value

#define RegTemp1 (0x4E)
/*
    7-4                 r   0000    unused

    3   TempMeasStart   w   0   Triggers the temperature measurement when set. Always reads 0.

    2   TempMeasRunning     r   0   Set to 1 while the temperature measurement is running. Toggles back to 0 when the measurement has completed.
                            The receiver can not be used while measuring temperature

    1-0                 r   01  unused
*/
#define RegTemp2 (0x4F)
//  7-0     TempValue       r       Measured temperature -1C per Lsb; Needs calibration for accuracy

//====================================

// Table 30 - Test Registers

// Name(Address)  Bits  Variable Name  Mode  Default  Description  Value

#define RegTestLna (0x58)
//  7-0     SensitivityBoost    rw  0x1B    High sensitivity or normal sensitivity mode: 0x1B = Normal mode; 0x2D = High sensitivity mode

#define RegTestPa1 (0x5A)
//  7-0     Pa13dBm1        rw  0x55    Set to 0x5D for +13 dBm operation. 0x55 = Normal mode and Rx mode; 0x5D  =  +13 dBm mode
//                            Revert to 0x55 when receiving or using PA0

#define RegTestPa2 (0x5C)
//  7-0     Pa13dBm2        rw  0x70    Set to 0x7C for +13 dBm operation; 0x70 = . Normal mode and Rx mode; 0x7C  = +13 dBm mode
//                            Revert to 0x70 when receiving or using PA0

#define RegTestDagc (0x6F)
/*
    7-0     ContinuousDagc  rw  0x30    *Fading Margin Improvement, refer to 3.4.4
                            0x00 =  Normal mode
                            0x20 =  Improved margin, use if AfcLowBetaOn=1
                            0x30 = Improved margin, use if AfcLowBetaOn=0
*/

#define RegTestAfc (0x71)
//  7-0     LowBetaAfcOffset    rw  0x00    AFC offset set for low modulation index systems, used if AfcLowBetaOn=1.
//                          Offset = LowBetaAfcOffset x 488 Hz

//========================================
//.end
